Hi

On Mon, Aug 05, 2013 at 08:21:48PM -0500, J Raynor wrote:
> I've attached a new patch that should support the same prefixes as neww.

I don't like calling cmd_get_default_path from session.c. How about this
instead? (Not tested much.)

I'll probably rename cmd_default_path* as soon as I think of good names,


diff --git a/cmd-new-session.c b/cmd-new-session.c
index 4eebe63..06cb5e3 100644
--- a/cmd-new-session.c
+++ b/cmd-new-session.c
@@ -35,9 +35,10 @@ enum cmd_retval       cmd_new_session_exec(struct cmd *, 
struct cmd_q *);
 
 const struct cmd_entry cmd_new_session_entry = {
        "new-session", "new",
-       "AdDF:n:Ps:t:x:y:", 0, 1,
-       "[-AdDP] [-F format] [-n window-name] [-s session-name] "
-       CMD_TARGET_SESSION_USAGE " [-x width] [-y height] [command]",
+       "Ac:dDF:n:Ps:t:x:y:", 0, 1,
+       "[-AdDP] [-c start-directory] [-F format] [-n window-name] "
+       "[-s session-name] " CMD_TARGET_SESSION_USAGE " [-x width] [-y height] "
+       "[command]",
        CMD_STARTSERVER|CMD_CANTNEST|CMD_SENDENVIRON,
        NULL,
        cmd_new_session_check,
@@ -139,6 +140,8 @@ cmd_new_session_exec(struct cmd *self, struct cmd_q *cmdq)
                else
                        cwd = "/";
        }
+       if (args_has(args, 'c'))
+               cwd = cmd_default_path0(cwd, cwd, args_get(args, 'c'));
 
        /* Find new session size. */
        if (c != NULL) {
diff --git a/cmd-new-window.c b/cmd-new-window.c
index cfc0b8b..22f38dd 100644
--- a/cmd-new-window.c
+++ b/cmd-new-window.c
@@ -103,7 +103,7 @@ cmd_new_window_exec(struct cmd *self, struct cmd_q *cmdq)
                cmd = options_get_string(&s->options, "default-command");
        else
                cmd = args->argv[0];
-       cwd = cmd_get_default_path(cmdq, args_get(args, 'c'));
+       cwd = cmd_default_path(cmdq, args_get(args, 'c'));
 
        if (idx == -1)
                idx = -1 - options_get_number(&s->options, "base-index");
diff --git a/cmd-split-window.c b/cmd-split-window.c
index 601dcb1..e9d8216 100644
--- a/cmd-split-window.c
+++ b/cmd-split-window.c
@@ -83,7 +83,7 @@ cmd_split_window_exec(struct cmd *self, struct cmd_q *cmdq)
                cmd = options_get_string(&s->options, "default-command");
        else
                cmd = args->argv[0];
-       cwd = cmd_get_default_path(cmdq, args_get(args, 'c'));
+       cwd = cmd_default_path(cmdq, args_get(args, 'c'));
 
        type = LAYOUT_TOPBOTTOM;
        if (args_has(args, 'h'))
diff --git a/tmux.h b/tmux.h
index 6ab9861..8652219 100644
--- a/tmux.h
+++ b/tmux.h
@@ -1767,7 +1767,10 @@ int               cmd_find_index(struct cmd_q *, const 
char *,
 struct winlink *cmd_find_pane(struct cmd_q *, const char *, struct session **,
                     struct window_pane **);
 char           *cmd_template_replace(const char *, const char *, int);
-const char             *cmd_get_default_path(struct cmd_q *, const char *);
+const char             *cmd_default_path(struct cmd_q *, const char *);
+const char             *cmd_default_path1(struct client *, struct session *,
+                    const char *);
+const char             *cmd_default_path0(const char *, const char *, const 
char *);
 extern const struct cmd_entry *cmd_table[];
 extern const struct cmd_entry cmd_attach_session_entry;
 extern const struct cmd_entry cmd_bind_key_entry;

------------------------------------------------------------------------------
Introducing Performance Central, a new site from SourceForge and 
AppDynamics. Performance Central is your source for news, insights, 
analysis and resources for efficient Application Performance Management. 
Visit us today!
http://pubads.g.doubleclick.net/gampad/clk?id=48897511&iu=/4140/ostg.clktrk
_______________________________________________
tmux-users mailing list
tmux-users@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/tmux-users

Reply via email to
diff --git a/cmd.c b/cmd.c
index 282fb11..fc34084 100644
--- a/cmd.c
+++ b/cmd.c
@@ -1279,33 +1279,56 @@ cmd_template_replace(const char *template, const char 
*s, int idx)
        return (buf);
 }
 
+/* Get default path with client from command queue. */
+const char *
+cmd_default_path(struct cmd_q *cmdq, const char *cwd)
+{
+       struct client   *c = cmdq->client;
+       struct session  *s;
+
+       if ((s = cmd_current_session(cmdq, 0)) == NULL)
+               return (NULL);
+
+       return (cmd_default_path1(c, s, cwd));
+}
+
+/* Get default path with client and session. */
+const char *
+cmd_default_path1(struct client *c, struct session *s, const char *cwd)
+{
+       const char      *this;
+
+       if (cwd == NULL)
+               cwd = options_get_string(&s->options, "default-path");
+
+       if (c != NULL && c->session == NULL && c->cwd != NULL)
+               this = c->cwd;
+       else if (s->curw != NULL)
+               this = osdep_get_cwd(s->curw->window->active->fd);
+       else
+               this = NULL;
+
+       return (cmd_default_path0(s->cwd, this, cwd));
+}
+
 /*
  * Return the default path for a new pane, using the given path or the
  * default-path option if it is NULL. Several special values are accepted: the
  * empty string or relative path for the current pane's working directory, ~
  * for the user's home, - for the session working directory, . for the tmux
  * server's working directory. The default on failure is the session's working
  * directory.
  */
 const char *
-cmd_get_default_path(struct cmd_q *cmdq, const char *cwd)
+cmd_default_path0(const char *base, const char *this, const char *cwd)
 {
-       struct client           *c = cmdq->client;
-       struct session          *s;
-       struct environ_entry    *envent;
        const char              *root;
+       struct environ_entry    *envent;
        char                     tmp[MAXPATHLEN];
        struct passwd           *pw;
        int                      n;
        size_t                   skip;
        static char              path[MAXPATHLEN];
 
-       if ((s = cmd_current_session(cmdq, 0)) == NULL)
-               return (NULL);
-
-       if (cwd == NULL)
-               cwd = options_get_string(&s->options, "default-path");
-
        skip = 1;
        if (strcmp(cwd, "$HOME") == 0 || strncmp(cwd, "$HOME/", 6) == 0) {
                /* User's home directory - $HOME. */
@@ -1315,8 +1338,8 @@ cmd_get_default_path(struct cmd_q *cmdq, const char *cwd)
                /* User's home directory - ~. */
                goto find_home;
        } else if (cwd[0] == '-' && (cwd[1] == '\0' || cwd[1] == '/')) {
-               /* Session working directory. */
-               root = s->cwd;
+               /* Base working directory. */
+               root = base;
                goto complete_path;
        } else if (cwd[0] == '.' && (cwd[1] == '\0' || cwd[1] == '/')) {
                /* Server working directory. */
@@ -1324,24 +1347,21 @@ cmd_get_default_path(struct cmd_q *cmdq, const char 
*cwd)
                        root = tmp;
                        goto complete_path;
                }
-               return (s->cwd);
+               return ("/");
        } else if (*cwd == '/') {
                /* Absolute path. */
                return (cwd);
        } else {
                /* Empty or relative path. */
-               if (c != NULL && c->session == NULL && c->cwd != NULL)
-                       root = c->cwd;
-               else if (s->curw != NULL)
-                       root = osdep_get_cwd(s->curw->window->active->fd);
+               if (this != NULL)
+                       root = this;
                else
-                       return (s->cwd);
+                       return (base);
                skip = 0;
-               if (root != NULL)
-                       goto complete_path;
+               goto complete_path;
        }
 
-       return (s->cwd);
+       return (base);
 
 find_home:
        envent = environ_find(&global_environ, "HOME");
@@ -1350,7 +1370,7 @@ find_home:
        else if ((pw = getpwuid(getuid())) != NULL)
                root = pw->pw_dir;
        else
-               return (s->cwd);
+               return (base);
 
 complete_path:
        if (root[skip] == '\0') {
@@ -1360,5 +1380,5 @@ complete_path:
        n = snprintf(path, sizeof path, "%s/%s", root, cwd + skip);
        if (n > 0 && (size_t)n < sizeof path)
                return (path);
-       return (s->cwd);
+       return (base);
 }
@@ -1279,33 +1279,56 @@ cmd_template_replace(const char *template, const char 
  * default-path option if it is NULL. Several special values are accepted: the
  * empty string or relative path for the current pane's working directory, ~
- * for the user's home, - for the session working directory, . for the tmux
- * server's working directory. The default on failure is the session's working
- * directory.
+ * for the user's home, - for the base working directory, . for the server
+ * working directory.
  */
 const char *